package algorithms.geneticAlgorithm.Components;

import java.util.ArrayList;
import java.util.Random;

public class GeneticAlgo<I extends GAIndividual> {

	public int popSize;
	public ArrayList<I> population;
	
	public GAIndividualFactory<I> randomFactory;
	public GASelectionStrategy<I> selector;
	public GACrossoverStrategy<I> reproducer;
	public GAMutationStrategy<I> mutator;
	public GAFittnessFunctionStrategy<I> fittnessFunction;
	
	public GeneticAlgo(GASelectionStrategy<I> selection, GACrossoverStrategy<I> crossover, GAMutationStrategy<I> mutator, 
			GAFittnessFunctionStrategy<I> fittnessFkt, GAIndividualFactory<I> factory, int popSize){
		
		this.selector = selection;
		this.reproducer = crossover;
		this.mutator = mutator;
		this.fittnessFunction = fittnessFkt;
		randomFactory = factory;
		this.popSize = popSize;
		
		population = new ArrayList<I>();
		
		generateRandomPopulation();
	}

	public void generateRandomPopulation() {
		for(int i = 0; i < popSize; i++){
			population.add(randomFactory.createRandomIndividual());
			fittnessFunction.calculateFittness(population.get(i));
		}
	}
	
	public void createNextGeneration(){
		ArrayList<I> nextGen = new ArrayList<I>();
		
		selector.setCurrentPopulation(population);
		while(nextGen.size() < popSize){
			ArrayList<I> freshInds = new ArrayList<I>();
			freshInds = reproducer.crossOver(selector.selectIndividuals());
			for(I i : freshInds){
				i = mutator.mutateIndividual(i);
				fittnessFunction.calculateFittness(i);
			}
			nextGen.addAll(freshInds);
		}
		if(popSize % 2 == 1){
			Random rnd = new Random();
			nextGen.remove(rnd.nextInt(popSize));
		}
		population = nextGen;
	}
	
	public ArrayList<I> getPopulation(){
		return population;
	}
}
